#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "log.h"

static dbi_conn connection;
static int sysloglevel = LOG_NOTICE;
static int mysqllevel = LOG_WARNING;
static int printlevel = LOG_WARNING;

void
open_log(dbi_conn conn, int sloglevel, int mlevel, int plevel)
{
	connection = conn;
	sysloglevel = sloglevel;
	mysqllevel = mlevel;
	printlevel = plevel;
	openlog(NULL, 0, LOG_USER);
}

void
log_handler(int priority, const char *handler, const char *msg, ...)
{
	char *message;
	va_list ap;
	va_start(ap, msg);
	if (vasprintf(&message, msg, ap) < 0)
		return;
	va_end(ap);
	if (connection && priority <= mysqllevel) {
		char *m, *h;
		dbi_conn_quote_string_copy(connection, message, &m);
		dbi_conn_quote_string_copy(connection, handler, &h);
		dbi_conn_queryf(connection,
				"INSERT INTO logs(handler, message) VALUES(%s, %s)",
				h, m);
		free(h);
		free(m);
	}
	log_msg(priority, "%s: %s", handler, message);
	free(message);
}

void
log_msg(int priority, const char *msg, ...)
{
	char *message;
	va_list ap;
	va_start(ap, msg);
	if (vasprintf(&message, msg, ap) < 0)
		return;
	va_end(ap);
	if (priority <= printlevel)
		printf("%s\n", message);
	if (priority <= sysloglevel)
		syslog(priority, "%s", message);
	free(message);
}
